@@ -1,6 +1,5 @@
 """
-NOTE: Buys are market orders since it does not matter anymore with coinbase fees.
-Sells are limits.
+NOTE: Buys are takers/market orders and sells are makers/limit orders
 """
 import sys
 import os
@@ -17,6 +16,12 @@ import cbpro
 from filelock import Timeout, FileLock
 from random import uniform
 
+os.environ['TZ'] = 'UTC'
+time.tzset()
+
+def parse_datetime(d):
+    return str(d).split('.')[0].split('Z')[0]
+
 def str2bool(v):
       return v.lower() in ("yes", "true", "t", "1")
 
@@ -52,6 +57,12 @@ CONF_DEFAULTS = {
     'debug': [
         ('debug_log_response', bool, False),
         ('debug_log_response_file', str, 'simplebot-debug.log'),
+    ],
+    'stoploss': [
+        ('stoploss_enable', bool, True),
+        ('stoploss_percent', Decimal, Decimal('-5.0')),
+        ('stoploss_seconds', int, 86400),
+        ('stoploss_strategy', str, 'report'),
     ]
 }
 
@@ -95,7 +106,8 @@ class SimpleCoinbaseBot:
         self.client = self.authenticate()
         self.wallet = None
         self.current_price = None
-        self.fee = None
+        self.fee_taker = None
+        self.fee_maker = None
         self.open_sells = []
         self.product_info = None
         self.min_size = None
@@ -110,12 +122,15 @@ class SimpleCoinbaseBot:
         self.get_all()
         self.__assert()
         self._open_cache()
-        self.logit('SimpleCoinbaseBot started: {} size-precision:{} usd-precision:{} current-fee:{} min-size:{} max-size:{}'.format(
-            self.coin, self.size_decimal_places, self.usd_decimal_places, self.fee, self.min_size, self.max_size
+        self.logit('SimpleCoinbaseBot started: {} size-precision:{} usd-precision:{} current-fees:{}/{} min-size:{} max-size:{}'.format(
+            self.coin, self.size_decimal_places, self.usd_decimal_places, self.fee_taker, self.fee_maker, self.min_size, self.max_size
         ))
         self.logit('SimpleCoinbaseBot started: {} sleep_seconds:{} sell_at_percent:{} max_sells_outstanding:{} max_buys_per_hour:{}'.format(
             self.coin, self.sleep_seconds, self.sell_at_percent, self.max_sells_outstanding, self.max_buys_per_hour
         ))
+        #self.stoploss_enable = True
+        #self.stoploss_percent = Decimal('-2.0')
+        #self.stoploss_seconds = 86400
 
     def _open_cache(self):
         if os.path.exists(self.cache_file):
@@ -200,9 +215,11 @@ class SimpleCoinbaseBot:
         fees = self.client._send_message('get', '/fees')
         assert('taker_fee_rate' in fees)
         self.logdebug(fees)
-        if Decimal(fees['taker_fee_rate']) > Decimal(fees['maker_fee_rate']):
-            return Decimal(fees['taker_fee_rate'])
-        return Decimal(fees['maker_fee_rate'])
+        self.fee_maker = Decimal(fees['maker_fee_rate'])
+        self.fee_taker = Decimal(fees['taker_fee_rate'])
+        #if self.fee_taker > self.fee_maker:
+        #    return Decimal(fees['taker_fee_rate'])
+        #return Decimal(fees['maker_fee_rate'])
 
     def _rand_msleep(self):
         time.sleep(uniform(0.1, 0.75))
@@ -215,7 +232,8 @@ class SimpleCoinbaseBot:
         self._rand_msleep()
         self.current_price = self.get_current_price()
         #self.open_sells = self.get_open_sells()
-        self.fee = self.get_fee()
+        #self.fee =
+        self.get_fee()
         self._rand_msleep()
         self.get_current_price_target()
         self._rand_msleep()
@@ -257,8 +275,10 @@ class SimpleCoinbaseBot:
         buy_amount = round(Decimal(self.buy_percent_of_wallet) * Decimal(self.wallet), self.usd_decimal_places)
         buy_size = round(Decimal(buy_amount)/self.current_price, self.size_decimal_places)
         if buy_size <= self.min_size:
-            self.logit('WARNING: Buy size is too small {} < {} wallet:{}.'.format(
-                buy_size, self.min_size, self.wallet))
+            self.logit('WARNING: Buy size is too small {} {} < {} wallet:{}.'.format(
+                buy_amount, buy_size, self.min_size, self.wallet))
+            #buy_size = self.min_size
+            self.logit('DEBUG: {}'.format(self.product_info))
             return
 
         # Check if USD wallet has enough available
@@ -276,7 +296,7 @@ class SimpleCoinbaseBot:
             buy_amount = self.buy_wallet_max
 
         # adjust size to fit with fee
-        buy_size = round(Decimal(buy_size) - Decimal(buy_size)*Decimal(self.fee), self.size_decimal_places)
+        buy_size = round(Decimal(buy_size) - Decimal(buy_size)*Decimal(self.fee_taker), self.size_decimal_places)
         self.logit('BUY: price:{} amount:{} size:{}'.format(
             self.current_price, buy_amount, buy_size))
         rc = self.client.place_market_order(
@@ -365,6 +385,67 @@ class SimpleCoinbaseBot:
             self.sendemail('BUY-ERROR', msg=msg)
         return buy
 
+    def run_stoploss(self, buy_order_id):
+        """ Cancel sell order, place new market sell to fill immediately
+            get response and update cache
+        """
+        v = self.cache[buy_order_id]
+        sell = v['sell_order']
+        # cancel
+        rc = self.client.cancel_order(sell['id'])
+        self.logdebug(rc)
+        self.logit('STOPLOSS: CANCEL-RESPONSE: {}'.format(rc))
+        time.sleep(5)
+    	# new order
+        rc = self.client.place_market_order(
+            product_id=self.coin,
+            side='sell',
+            size=sell['size']
+        )
+        self.logdebug(rc)
+        self.cache[buy_order_id]['sell_order'] = rc
+        self._write_cache()
+        self.logit('STOPLOSS: SELL-RESPONSE: {}'.format(rc))
+        order_id = rc['id']
+        time.sleep(5)
+        done = False
+        while 1:
+            try:
+                status = self.client.get_order(order_id)
+                self.logdebug(status)
+                self.cache[buy_order_id]['sell_order'] = status
+                self._write_cache()
+                if 'settled' in status:
+                    if status['settled']:
+                        self.logit('SELL-FILLED: {}'.format(status))
+                        self.cache[buy_order_id]['sell_order_completed'] = status
+                        self._write_cache()
+                        done = True
+                        break
+                else:
+                    if 'message' in status:
+                        self.logit('WARNING: Failed to get order status: {}'.format(status['message']))
+                        self.logit('WARNING: Order status failure may be temporary, due to coinbase issues or exchange delays. Check: https://status.pro.coinbase.com')
+                        status_errors += 1
+                    else:
+                        self.logit('WARNING: Failed to get order status: {}'.format(order_id))
+                        status_errors += 1
+                    time.sleep(10)
+                if status_errors > 10:
+                    errors += 1
+            except Exception as err:
+                self.logit('WARNING: get_order() failed:', err)
+                errors += 1
+                time.sleep(8)
+            if errors > 5:
+                self.logit('WARNING: Failed to get order. Manual intervention needed.: {}'.format(
+                    order_id))
+                break
+            time.sleep(2)
+
+        if not done:
+            self.logit('ERROR: Failed to get_order() for stoploss. This is an error and TODO item on how to handle')
+
     def check_sell_orders(self):
         """ Check if any sell orders have completed """
         for buy_order_id, v in self.cache.items():
@@ -379,6 +460,12 @@ class SimpleCoinbaseBot:
                     self.cache[buy_order_id]['completed'] = True
                     self._write_cache()
                 continue
+            if 'message' in v['sell_order']:
+                self.logit('WARNING: Corrupted sell order, marking as done: {}'.format(v['sell_order']))
+                self.cache[buy_order_id]['completed'] = True
+                self.cache[buy_order_id]['sell_order'] = None
+                self._write_cache()
+                continue
             sell = self.client.get_order(v['sell_order']['id'])
             if 'message' in sell:
                 self.logit('WARNING: Failed to get sell order status (retrying later): {}'.format(
@@ -402,9 +489,10 @@ class SimpleCoinbaseBot:
                     buy_value = Decimal(v['last_status']['executed_value'])
                     #buy_sell_diff = round((sell_price*sell_filled_size) - (buy_price*buy_filled_size), 2)
                     buy_sell_diff = round(sell_value - buy_value, 2)
+                    done_at = time.mktime(time.strptime(parse_datetime(sell['done_at']), '%Y-%m-%dT%H:%M:%S'))
                     self.cache[buy_order_id]['profit_usd'] = buy_sell_diff
                     msg = 'SELL-COMPLETED: ~duration:{:.2f} bought_val:{} sold_val:{} profit_usd:{}'.format(
-                        time.time() - v['time'],
+                        time.time() - done_at,
                         round(buy_value, 2),
                         round(sell_value, 2),
                         buy_sell_diff
@@ -414,10 +502,39 @@ class SimpleCoinbaseBot:
                 else:
                     self.logit('SELL-COMPLETED-WITH-OTHER-STATUS: {}'.format(sell['status']))
                 self._write_cache()
+            else:
+                # check for stoploss if enabled
+                if self.stoploss_enable:
+                    created_at = time.mktime(time.strptime(parse_datetime(sell['created_at']), '%Y-%m-%dT%H:%M:%S'))
+                    duration = time.time() - created_at
+                    bought_price = round(Decimal(v['last_status']['executed_value']) / Decimal(v['last_status']['filled_size']), 4)
+                    p = 100*(self.current_price/bought_price) - Decimal('100.0')
+                    stop_seconds = False
+                    stop_percent = False
+                    if duration >= self.stoploss_seconds:
+                        stop_seconds = True
+                    if p <= self.stoploss_percent:
+                        stop_percent = True
+                    if (stop_seconds or stop_percent) and self.stoploss_strategy == 'report':
+                        self.logit('STOPLOSS: percent:{} duration:{}'.format(p, duration))
+
+                    if self.stoploss_strategy == 'both' and stop_percent and stop_seconds:
+                        self.logit('STOPLOSS: running stoploss strategy: {} percent:{} duration:{}'.format(
+                            self.stoploss_strategy,
+                            p, duration
+                        ))
+                        self.run_stoploss(buy_order_id)
+                    elif self.stoploss_strategy == 'either' and (stop_percent or stop_seconds):
+                        self.logit('STOPLOSS: running stoploss strategy: {} percent:{} duration:{}'.format(
+                            self.stoploss_strategy,
+                            p, duration
+                        ))
+                        self.run_stoploss(buy_order_id)
+
             time.sleep(0.75)
 
     def get_current_price_target(self):
-        current_percent_increase = (self.fee*2)+(self.sell_at_percent/100)
+        current_percent_increase = (self.fee_maker+self.fee_taker)+(self.sell_at_percent/100)
         self.current_price_target = round(
             self.current_price * current_percent_increase + self.current_price,
             self.usd_decimal_places
@@ -468,10 +585,17 @@ class SimpleCoinbaseBot:
             sell_order = v['sell_order']
             if not sell_order:
                 continue
-            sell_price = Decimal(sell_order['price'])
-            adjusted_sell_price = round(sell_price - (self.fee*2*sell_price), self.usd_decimal_places)
-            if adjusted_sell_price <= self.current_price_target:
-                can = False
+            if not 'price' in sell_order:
+                pass
+                #iself.logit('WARNING: Corrupted sell order. Writing as completed (error): {}'.format(sell_order))
+                #self.cache[buy_order_id]['sell_order'] = None
+                #self.cache[buy_order_id]['completed'] = True
+                #self._write_cache()
+            else:
+                sell_price = Decimal(sell_order['price'])
+                adjusted_sell_price = round(sell_price - ((self.fee_maker+self.fee_taker)*sell_price), self.usd_decimal_places)
+                if adjusted_sell_price <= self.current_price_target:
+                    can = False
         return can
 
     def run(self):
@@ -479,8 +603,8 @@ class SimpleCoinbaseBot:
         time.sleep(uniform(1, 5))
         while 1:
             self.get_all()
-            self.logit('STATUS: price:{} fee:{} wallet:{} open-sells:{} price-target:{} can-buy:{}'.format(
-                self.current_price, self.fee, self.wallet, self.total_open_orders, self.current_price_target,
+            self.logit('STATUS: price:{} fees:{}/{} wallet:{} open-sells:{} price-target:{} can-buy:{}'.format(
+                self.current_price, self.fee_taker, self.fee_maker, self.wallet, self.total_open_orders, self.current_price_target,
                 self.can_buy,
             ))
             self.maybe_buy_sell()
