@@ -19,6 +19,8 @@ from operator import getitem
 from .color import colors
 from .getch import getch, getchb
 from .termsize import get_terminal_size
+import cbpro
+
 os.environ['TZ'] = 'UTC'
 time.tzset()
 
@@ -27,13 +29,45 @@ g_last_input = None
 g_filter = ''
 g_paused = False
 g_show_orders = False
+g_print_buf = ''
+
+PRICE_CACHE = {}
+PRICE_CACHE_RATE = 73.1331 #update every N seconds
+
+def parse_datetime(d):
+    return str(d).split('.')[0].split('Z')[0]
+
+def print_b(msg):
+    global g_print_buf
+    g_print_buf = '{}{}\n'.format(g_print_buf, msg)
 
 def clear():
+    #global g_print_buf
+    #g_print_buf = ''
     if name == 'nt':
         system('cls')
     else:
         system('clear')
 
+def get_current_price(coin):
+    last_update = time.time()
+    current_price = Decimal('0.0')
+    if not coin in PRICE_CACHE:
+        public_client = cbpro.PublicClient()
+        ticker = public_client.get_product_ticker(product_id=coin)
+        current_price = Decimal(ticker['price'])
+    else:
+        # check cache age
+        if time.time() - PRICE_CACHE[coin]['last_update'] > PRICE_CACHE_RATE:
+            public_client = cbpro.PublicClient()
+            ticker = public_client.get_product_ticker(product_id=coin)
+            current_price = Decimal(ticker['price'])
+        else:
+            last_update = PRICE_CACHE[coin]['last_update']
+            current_price  = PRICE_CACHE[coin]['price']
+    PRICE_CACHE[coin] = {'price':current_price, 'last_update':last_update}
+    return Decimal(current_price)
+
 def get_input():
     global g_last_input
     global g_paused
@@ -149,12 +183,12 @@ def show_orders():
         colors.fg.red, colors.reset, colors.fg.lightgrey,
         colors.reset, colors.fg.pink, colors.reset, colors.fg.lightgrey,
         colors.reset, colors.fg.green, colors.reset, colors.fg.lightgrey,
-        str(datetime.now()).split('.')[0])
+        parse_datetime(datetime.now()))
     )
     draw_line(1)
     print('Open orders:')
-    print('{:>17} {:>9} {:>16} {:>16} {:>18}'.format(
-        'Duration', 'Coin', 'Bought', 'Sell-Price', 'Size',
+    print('{:>15} {:>9} {:>13} {:>13} {:>14} {:>11}'.format(
+        'Duration', 'Coin', 'Bought', 'Sell-Price', 'Size', 'Diff',
     ))
     w, h = get_terminal_size()
     hn = 0
@@ -181,17 +215,20 @@ def show_orders():
             if v['completed'] or not v['sell_order']:
                 continue
             sell = v['sell_order']
-            created_at = time.mktime(time.strptime(sell['created_at'].split('.')[0], '%Y-%m-%dT%H:%M:%S'))
+            created_at = time.mktime(time.strptime(parse_datetime(sell['created_at']), '%Y-%m-%dT%H:%M:%S'))
             duration = cur_time - created_at
             price = sell['price']
             size = sell['size']
             bought_price = round(Decimal(v['last_status']['executed_value']) / Decimal(v['last_status']['filled_size']), 4)
             if hn+10 < h:
-                print('{:>17} {:>9} {:>16} {:>16} {:>18}'.format(
+                cur_price = get_current_price(sell['product_id'])
+                #print('{:>17} {:>9} {:>16} {:>16} {:>18}'.format(
+                print('{:>15} {:>9} {:>13} {:>13} {:>14} {:>11}'.format(
                     sec2time(duration),
                     sell['product_id'],
                     bought_price, price,
                     size,
+                    round(cur_price - bought_price, 2)
                 ))
             hn += 1
     if hn > h:
@@ -215,6 +252,7 @@ def top():
     profit_dates = {}
     w, h = get_terminal_size()
     hn = 0
+    open_percents = []
     for f in files:
         data = None
         coin = None
@@ -236,13 +274,15 @@ def top():
                     'open_orders':0, 'done_orders':0, 'avg_close_time':0.0, 'error_orders':0,
                 }
             first_status = v['first_status']
-            epoch = time.mktime(time.strptime(first_status['created_at'].split('.')[0], '%Y-%m-%dT%H:%M:%S'))
-            if v['completed'] and 'sell_order_completed' in v and v['sell_order_completed']:
+            epoch = time.mktime(time.strptime(parse_datetime(first_status['created_at']), '%Y-%m-%dT%H:%M:%S'))
+            if v['completed'] and 'sell_order_completed' in v and v['sell_order_completed'] and v['profit_usd']:
+                # NOTE: Getting some completed orders w/o all the information filled in (done_at)
+                # This seems to happen when failing to retreive status in the bot code
                 date_only = v['sell_order_completed']['done_at'].split('T')[0]
                 if not date_only in profit_dates:
                     profit_dates[date_only] = []
                 profit_dates[date_only].append(v['profit_usd'])
-                end_epoch = time.mktime(time.strptime(v['sell_order_completed']['done_at'].split('.')[0], '%Y-%m-%dT%H:%M:%S'))
+                end_epoch = time.mktime(time.strptime(parse_datetime(v['sell_order_completed']['done_at']), '%Y-%m-%dT%H:%M:%S'))
                 epoch_diff = end_epoch - epoch
                 cur_diff = cur_time - end_epoch
                 if cur_diff < (86400/12):
@@ -255,7 +295,16 @@ def top():
             elif v['completed']:
                 stats[coin]['error_orders'] += 1
             else:
-                start_epoch = time.mktime(time.strptime(v['first_status']['created_at'].split('.')[0], '%Y-%m-%dT%H:%M:%S'))
+                cur_price = get_current_price(coin)
+                try:
+                    cur_perc = (100*(cur_price/Decimal(v['sell_order']['price']))) - Decimal('100.0')
+                    open_percents.append(cur_perc)
+                except Exception as err:
+                    # I think sometimes the state drifts after a cancel
+                    # and v['sell'] was removed but v['completed'] is not True yet
+                    #print('ERR:', err, v['sell_order'])
+                    pass
+                start_epoch = time.mktime(time.strptime(parse_datetime(v['first_status']['created_at']), '%Y-%m-%dT%H:%M:%S'))
                 open_times.append(cur_time - start_epoch)
                 stats[coin]['open_orders'] += 1
 
@@ -266,8 +315,8 @@ def top():
         colors.fg.red, colors.reset, colors.fg.lightgrey,
         colors.reset, colors.fg.pink, colors.reset, colors.fg.lightgrey,
         colors.reset, colors.fg.blue, colors.reset, colors.fg.lightgrey,
-        str(datetime.now()).split('.')[0])
-    )
+        parse_datetime(datetime.now())
+    ))
     draw_line(1)
     print('{}{:>8} {}{:>13} {}{:>7} {}{:>7} {}{:>7} {}{:>12} {}{:>19}{}'.format(
         colors.fg.lightcyan, 'Coin',
@@ -339,7 +388,13 @@ def top():
         print('{}{:>16} {:>16} {:>16} {:>16}'.format(colors.fg.lightgrey, 'Open order times', 'Min', 'Max', 'Avg'))
         print('{}{:>16} {:>16} {:>16} {:>16}'.format(colors.fg.lightred, ' ', min_open_time, max_open_time, avg_open_time))
     hn += 2
-
+    if hn+10 < h:
+        cur_drift = round(avg(open_percents), 2)
+        if cur_drift < 0:
+            print('{}Avg-drift: {}{}%'.format(colors.reset, colors.fg.red, cur_drift))
+        else:
+            print('{}Avg-drift: {}{}%'.format(colors.reset, colors.fg.green, cur_drift))
+    hn+=1
     # Last 7 days with profits
     #print('{}{:>26}'.format(colors.fg.lightgrey, 'Last 7 days profits'))
     sorted_dates_val = OrderedDict(sorted(profit_dates.items(), key = lambda x: x[1], reverse=True))
@@ -359,6 +414,7 @@ def top():
         draw_line(3)
     hn += 1
     if y:
+        total_profit = []
         max_y = max(y)
         width = 50
         for i in range(len(y)):
@@ -368,13 +424,17 @@ def top():
             if hn+10 < h:
                 print('{}{}{}{:>11} {}{}'.format(colors.fg.cyan, key, colors.fg.green, '$'+str(yy), colors.fg.darkgrey, '*'*nstars))
             hn += 1
+            total_profit.append(yy)
+        if hn+10 < h:
+            nstars = int((avg(total_profit)/max_y) * width)
+            print('{}{}{}{:>11} {}{}^'.format(colors.fg.cyan, 'Daily Avg ', colors.fg.green, '$'+str(round(avg(total_profit), 2)), colors.fg.darkgrey, ' '*(nstars-1)))
     if recent:
         if hn+10 < h:
             draw_line(3)
-            print('{}{}'.format(colors.fg.lightgrey, 'Recently completed orders'), colors.fg.blue)
+            print('{}{}{}'.format(colors.fg.lightgrey, 'Recently completed orders', colors.fg.blue))
         hn += 2
         #print('Recently completed orders:', colors.fg.lightblue)
-        print('    {:>8} {:>11} {:>17} {:>19}'.format('Coin', 'Profit', 'Duration', 'Completed'), colors.fg.green)
+        print('    {:>8} {:>11} {:>17} {:>19}{}'.format('Coin', 'Profit', 'Duration', 'Completed', colors.fg.green))
         # bubble sort, why not
         for i in range(len(recent)-1):
             for j in range(len(recent)-i-1):
@@ -387,8 +447,8 @@ def top():
             if not re.search(g_filter, coin, re.IGNORECASE):
                 continue
             first_status = v['first_status']
-            epoch = time.mktime(time.strptime(first_status['created_at'].split('.')[0], '%Y-%m-%dT%H:%M:%S'))
-            end_epoch = time.mktime(time.strptime(v['sell_order_completed']['done_at'].split('.')[0], '%Y-%m-%dT%H:%M:%S'))
+            epoch = time.mktime(time.strptime(parse_datetime(first_status['created_at']), '%Y-%m-%dT%H:%M:%S'))
+            end_epoch = time.mktime(time.strptime(parse_datetime(v['sell_order_completed']['done_at']), '%Y-%m-%dT%H:%M:%S'))
             epoch_diff = end_epoch - epoch
             cur_diff = cur_time - end_epoch
             profit = round(v['profit_usd'], 2)
@@ -408,6 +468,7 @@ def main():
     global g_paused
     global g_filter
     global g_show_orders
+    global g_print_buf
     if len(sys.argv) != 2:
         usage()
     files = glob.glob(sys.argv[1]+'/*.cache')
@@ -420,6 +481,7 @@ def main():
     sleep_time = 1.33
     running = True
     while running:
+        #clear()
         while g_paused:
             time.sleep(0.1)
         with GETCH_LOCK:
